import asyncio
from random import choice, randint
import re

import niobot
from niobot import (
    CommandParserError,
    Context,
    Module,
    NioBot,
)

from .stubs import ERROR_STARTS, get_random_stub, format_mention


_DICE_LOCATIONS = [
    "pulls some dice seemingly from thin air.",
    "extracts some dice from his mouth.",
    "spins around and a number of dice appear in his hands.",
    "opens the closest drawer and takes out a container of random dice.",
    "powers on a tablet and executes a dice-rolling application.",
]


class ChanceModule(Module):
    """Commands that simulate chance (Dice rolls, card selection, etc)."""

    def __init__(self, bot: NioBot):
        self.bot = bot

    @niobot.command()
    async def roll(self, ctx: Context, dice: str):
        """Roll some dice with an optional modifier."""
        err_prelude = get_random_stub(ERROR_STARTS, ctx.message.sender)
        valid = re.match(r"(\d+)d(\d+)([\+-/\*]\d+)?", dice)
        if valid is None:
            await ctx.client.send_message(
                ctx.room,
                f"{err_prelude}that's an invalid dice roll.",
                reply_to=ctx.message,
                message_type="m.text",
                content_type="html",
            )
            raise CommandParserError("Invalid dice roll notation.")
        dice_parts = (int(valid.group(1)), int(valid.group(2)), valid.group(3))

        if not 1 <= dice_parts[0] <= 100:
            await ctx.client.send_message(
                ctx.room,
                f"{err_prelude}I can only roll between 1 and 100 dice.",
                reply_to=ctx.message,
                message_type="m.text",
                content_type="html",
            )
            raise CommandParserError("Can only roll between 1 and 100 dice.")

        if not 2 <= dice_parts[1] <= 100:
            await ctx.client.send_message(
                ctx.room,
                f"{err_prelude}the dice can only have between 2 and 100 faces.",
                reply_to=ctx.message,
                message_type="m.text",
                content_type="html",
            )
            raise CommandParserError("Dice must have between 2 and 100 faces.")

        rolls: list[int] = []
        for _ in range(dice_parts[0]):
            rolls.append(randint(1, dice_parts[1]))
        total = sum(rolls)

        await ctx.client.send_message(
            ctx.room,
            choice(_DICE_LOCATIONS),
            message_type="m.emote",
            content_type="html",
        )

        # Give some delay to allow for reading the emote.
        await asyncio.sleep(1)

        html_sender = format_mention(ctx.message.sender)
        await ctx.client.send_message(
            ctx.room,
            f"{html_sender}: {dice} = {total} ({rolls})",
            reply_to=ctx.message,
            message_type="m.text",
            content_type="html",
        )
